// ------------------------------------------------------------------------------
//  <autogenerated>
//      This code was generated by a tool.
//      Mono Runtime Version: 2.0.50727.42
// 
//      Changes to this file may cause incorrect behavior and will be lost if 
//      the code is regenerated.
//  </autogenerated>
// ------------------------------------------------------------------------------



public partial class MainWindow {
    
    private Gtk.Action File;
    
    private Gtk.Action Open;
    
    private Gtk.Action Open1;
    
    private Gtk.Action Export;
    
    private Gtk.Action SaveAs;
    
    private Gtk.Action Import;
    
    private Gtk.Action View;
    
    private Gtk.Action ZoomIn;
    
    private Gtk.Action ZoomOut;
    
    private Gtk.VBox vbox3;
    
    private Gtk.MenuBar menubar3;
    
    private Gtk.HPaned paned;
    
    private Gtk.Label geglViewDummy;
    
    private Gtk.VBox controlsVBox;
    
    private Gtk.Table table1;
    
    private Gtk.HScale blueScale;
    
    private Gtk.HScale greenScale;
    
    private Gtk.Label label2;
    
    private Gtk.Label label3;
    
    private Gtk.Label label6;
    
    private Gtk.HScale redScale;
    
    private Gtk.Label curveDummy;
    
    private Gtk.Label statusLabel;
    
    private Gtk.HSeparator hseparator1;
    
    private Gtk.Table table2;
    
    private Gtk.Label label4;
    
    private Gtk.Label label5;
    
    private Gtk.HScale tintAmountScale;
    
    private Gtk.HScale tintHueScale;
    
    protected virtual void Build() {
        Stetic.Gui.Initialize();
        // Widget MainWindow
        Gtk.UIManager w1 = new Gtk.UIManager();
        Gtk.ActionGroup w2 = new Gtk.ActionGroup("Default");
        this.File = new Gtk.Action("File", Mono.Unix.Catalog.GetString("File"), null, null);
        this.File.ShortLabel = Mono.Unix.Catalog.GetString("File");
        w2.Add(this.File, null);
        this.Open = new Gtk.Action("Open", Mono.Unix.Catalog.GetString("Open..."), null, null);
        this.Open.ShortLabel = Mono.Unix.Catalog.GetString("Open...");
        w2.Add(this.Open, null);
        this.Open1 = new Gtk.Action("Open1", Mono.Unix.Catalog.GetString("Open..."), null, null);
        this.Open1.ShortLabel = Mono.Unix.Catalog.GetString("Open...");
        w2.Add(this.Open1, null);
        this.Export = new Gtk.Action("Export", Mono.Unix.Catalog.GetString("Export..."), null, null);
        this.Export.ShortLabel = Mono.Unix.Catalog.GetString("Export...");
        w2.Add(this.Export, null);
        this.SaveAs = new Gtk.Action("SaveAs", Mono.Unix.Catalog.GetString("Save as..."), null, null);
        this.SaveAs.ShortLabel = Mono.Unix.Catalog.GetString("Save as...");
        w2.Add(this.SaveAs, null);
        this.Import = new Gtk.Action("Import", Mono.Unix.Catalog.GetString("Import..."), null, null);
        this.Import.ShortLabel = Mono.Unix.Catalog.GetString("Import...");
        w2.Add(this.Import, null);
        this.View = new Gtk.Action("View", Mono.Unix.Catalog.GetString("View"), null, null);
        this.View.ShortLabel = Mono.Unix.Catalog.GetString("View");
        w2.Add(this.View, null);
        this.ZoomIn = new Gtk.Action("ZoomIn", Mono.Unix.Catalog.GetString("Zoom in"), null, null);
        this.ZoomIn.ShortLabel = Mono.Unix.Catalog.GetString("Zoom in");
        w2.Add(this.ZoomIn, "plus");
        this.ZoomOut = new Gtk.Action("ZoomOut", Mono.Unix.Catalog.GetString("Zoom out"), null, null);
        this.ZoomOut.ShortLabel = Mono.Unix.Catalog.GetString("Zoom out");
        w2.Add(this.ZoomOut, "minus");
        w1.InsertActionGroup(w2, 0);
        this.AddAccelGroup(w1.AccelGroup);
        this.Name = "MainWindow";
        this.Title = Mono.Unix.Catalog.GetString("BW Processor");
        this.WindowPosition = ((Gtk.WindowPosition)(4));
        // Container child MainWindow.Gtk.Container+ContainerChild
        this.vbox3 = new Gtk.VBox();
        this.vbox3.Name = "vbox3";
        this.vbox3.Spacing = 6;
        // Container child vbox3.Gtk.Box+BoxChild
        w1.AddUiFromString("<ui><menubar name='menubar3'><menu action='File'><menuitem action='Open1'/><menuitem action='SaveAs'/><menuitem action='Import'/><menuitem action='Export'/></menu><menu action='View'><menuitem action='ZoomIn'/><menuitem action='ZoomOut'/></menu></menubar></ui>");
        this.menubar3 = ((Gtk.MenuBar)(w1.GetWidget("/menubar3")));
        this.menubar3.Name = "menubar3";
        this.vbox3.Add(this.menubar3);
        Gtk.Box.BoxChild w3 = ((Gtk.Box.BoxChild)(this.vbox3[this.menubar3]));
        w3.Position = 0;
        w3.Expand = false;
        w3.Fill = false;
        // Container child vbox3.Gtk.Box+BoxChild
        this.paned = new Gtk.HPaned();
        this.paned.CanFocus = true;
        this.paned.Name = "paned";
        this.paned.Position = 708;
        // Container child paned.Gtk.Paned+PanedChild
        this.geglViewDummy = new Gtk.Label();
        this.geglViewDummy.Name = "geglViewDummy";
        this.geglViewDummy.LabelProp = Mono.Unix.Catalog.GetString("label1");
        this.paned.Add(this.geglViewDummy);
        Gtk.Paned.PanedChild w4 = ((Gtk.Paned.PanedChild)(this.paned[this.geglViewDummy]));
        w4.Resize = false;
        // Container child paned.Gtk.Paned+PanedChild
        this.controlsVBox = new Gtk.VBox();
        this.controlsVBox.Name = "controlsVBox";
        this.controlsVBox.Spacing = 6;
        // Container child controlsVBox.Gtk.Box+BoxChild
        this.table1 = new Gtk.Table(((uint)(3)), ((uint)(2)), false);
        this.table1.Name = "table1";
        this.table1.RowSpacing = ((uint)(6));
        this.table1.ColumnSpacing = ((uint)(6));
        // Container child table1.Gtk.Table+TableChild
        this.blueScale = new Gtk.HScale(null);
        this.blueScale.CanFocus = true;
        this.blueScale.Name = "blueScale";
        this.blueScale.Adjustment.Lower = -200;
        this.blueScale.Adjustment.Upper = 200;
        this.blueScale.Adjustment.PageIncrement = 10;
        this.blueScale.Adjustment.StepIncrement = 1;
        this.blueScale.Adjustment.Value = 30;
        this.blueScale.DrawValue = true;
        this.blueScale.Digits = 0;
        this.blueScale.ValuePos = ((Gtk.PositionType)(2));
        this.table1.Add(this.blueScale);
        Gtk.Table.TableChild w5 = ((Gtk.Table.TableChild)(this.table1[this.blueScale]));
        w5.TopAttach = ((uint)(2));
        w5.BottomAttach = ((uint)(3));
        w5.LeftAttach = ((uint)(1));
        w5.RightAttach = ((uint)(2));
        w5.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table1.Gtk.Table+TableChild
        this.greenScale = new Gtk.HScale(null);
        this.greenScale.CanFocus = true;
        this.greenScale.Name = "greenScale";
        this.greenScale.Adjustment.Lower = -200;
        this.greenScale.Adjustment.Upper = 200;
        this.greenScale.Adjustment.PageIncrement = 10;
        this.greenScale.Adjustment.StepIncrement = 1;
        this.greenScale.Adjustment.Value = 30;
        this.greenScale.DrawValue = true;
        this.greenScale.Digits = 0;
        this.greenScale.ValuePos = ((Gtk.PositionType)(2));
        this.table1.Add(this.greenScale);
        Gtk.Table.TableChild w6 = ((Gtk.Table.TableChild)(this.table1[this.greenScale]));
        w6.TopAttach = ((uint)(1));
        w6.BottomAttach = ((uint)(2));
        w6.LeftAttach = ((uint)(1));
        w6.RightAttach = ((uint)(2));
        w6.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table1.Gtk.Table+TableChild
        this.label2 = new Gtk.Label();
        this.label2.Name = "label2";
        this.label2.Xalign = 1F;
        this.label2.LabelProp = Mono.Unix.Catalog.GetString("Green");
        this.table1.Add(this.label2);
        Gtk.Table.TableChild w7 = ((Gtk.Table.TableChild)(this.table1[this.label2]));
        w7.TopAttach = ((uint)(1));
        w7.BottomAttach = ((uint)(2));
        w7.XOptions = ((Gtk.AttachOptions)(4));
        w7.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table1.Gtk.Table+TableChild
        this.label3 = new Gtk.Label();
        this.label3.Name = "label3";
        this.label3.Xalign = 1F;
        this.label3.LabelProp = Mono.Unix.Catalog.GetString("Blue");
        this.table1.Add(this.label3);
        Gtk.Table.TableChild w8 = ((Gtk.Table.TableChild)(this.table1[this.label3]));
        w8.TopAttach = ((uint)(2));
        w8.BottomAttach = ((uint)(3));
        w8.XOptions = ((Gtk.AttachOptions)(4));
        w8.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table1.Gtk.Table+TableChild
        this.label6 = new Gtk.Label();
        this.label6.Name = "label6";
        this.label6.Xalign = 1F;
        this.label6.LabelProp = Mono.Unix.Catalog.GetString("Red");
        this.table1.Add(this.label6);
        Gtk.Table.TableChild w9 = ((Gtk.Table.TableChild)(this.table1[this.label6]));
        w9.XOptions = ((Gtk.AttachOptions)(4));
        w9.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table1.Gtk.Table+TableChild
        this.redScale = new Gtk.HScale(null);
        this.redScale.CanFocus = true;
        this.redScale.Name = "redScale";
        this.redScale.Adjustment.Lower = -200;
        this.redScale.Adjustment.Upper = 200;
        this.redScale.Adjustment.PageIncrement = 10;
        this.redScale.Adjustment.StepIncrement = 1;
        this.redScale.Adjustment.Value = 40;
        this.redScale.DrawValue = true;
        this.redScale.Digits = 0;
        this.redScale.ValuePos = ((Gtk.PositionType)(2));
        this.table1.Add(this.redScale);
        Gtk.Table.TableChild w10 = ((Gtk.Table.TableChild)(this.table1[this.redScale]));
        w10.LeftAttach = ((uint)(1));
        w10.RightAttach = ((uint)(2));
        w10.YOptions = ((Gtk.AttachOptions)(4));
        this.controlsVBox.Add(this.table1);
        Gtk.Box.BoxChild w11 = ((Gtk.Box.BoxChild)(this.controlsVBox[this.table1]));
        w11.Position = 0;
        w11.Expand = false;
        w11.Fill = false;
        // Container child controlsVBox.Gtk.Box+BoxChild
        this.curveDummy = new Gtk.Label();
        this.curveDummy.Name = "curveDummy";
        this.curveDummy.LabelProp = Mono.Unix.Catalog.GetString("label2");
        this.controlsVBox.Add(this.curveDummy);
        Gtk.Box.BoxChild w12 = ((Gtk.Box.BoxChild)(this.controlsVBox[this.curveDummy]));
        w12.Position = 1;
        // Container child controlsVBox.Gtk.Box+BoxChild
        this.statusLabel = new Gtk.Label();
        this.statusLabel.Name = "statusLabel";
        this.statusLabel.Xalign = 0F;
        this.statusLabel.LabelProp = Mono.Unix.Catalog.GetString("Label");
        this.controlsVBox.Add(this.statusLabel);
        Gtk.Box.BoxChild w13 = ((Gtk.Box.BoxChild)(this.controlsVBox[this.statusLabel]));
        w13.PackType = ((Gtk.PackType)(1));
        w13.Position = 2;
        w13.Expand = false;
        w13.Fill = false;
        // Container child controlsVBox.Gtk.Box+BoxChild
        this.hseparator1 = new Gtk.HSeparator();
        this.hseparator1.Name = "hseparator1";
        this.controlsVBox.Add(this.hseparator1);
        Gtk.Box.BoxChild w14 = ((Gtk.Box.BoxChild)(this.controlsVBox[this.hseparator1]));
        w14.PackType = ((Gtk.PackType)(1));
        w14.Position = 3;
        w14.Expand = false;
        w14.Fill = false;
        // Container child controlsVBox.Gtk.Box+BoxChild
        this.table2 = new Gtk.Table(((uint)(2)), ((uint)(2)), false);
        this.table2.Name = "table2";
        this.table2.RowSpacing = ((uint)(6));
        this.table2.ColumnSpacing = ((uint)(6));
        // Container child table2.Gtk.Table+TableChild
        this.label4 = new Gtk.Label();
        this.label4.Name = "label4";
        this.label4.Xalign = 1F;
        this.label4.LabelProp = Mono.Unix.Catalog.GetString("Tint Hue");
        this.table2.Add(this.label4);
        Gtk.Table.TableChild w15 = ((Gtk.Table.TableChild)(this.table2[this.label4]));
        w15.XOptions = ((Gtk.AttachOptions)(4));
        w15.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table2.Gtk.Table+TableChild
        this.label5 = new Gtk.Label();
        this.label5.Name = "label5";
        this.label5.Xalign = 1F;
        this.label5.LabelProp = Mono.Unix.Catalog.GetString("Tint Amount");
        this.table2.Add(this.label5);
        Gtk.Table.TableChild w16 = ((Gtk.Table.TableChild)(this.table2[this.label5]));
        w16.TopAttach = ((uint)(1));
        w16.BottomAttach = ((uint)(2));
        w16.XOptions = ((Gtk.AttachOptions)(4));
        w16.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table2.Gtk.Table+TableChild
        this.tintAmountScale = new Gtk.HScale(null);
        this.tintAmountScale.CanFocus = true;
        this.tintAmountScale.Name = "tintAmountScale";
        this.tintAmountScale.Adjustment.Upper = 100;
        this.tintAmountScale.Adjustment.PageIncrement = 10;
        this.tintAmountScale.Adjustment.StepIncrement = 1;
        this.tintAmountScale.Adjustment.Value = 8;
        this.tintAmountScale.DrawValue = true;
        this.tintAmountScale.Digits = 0;
        this.tintAmountScale.ValuePos = ((Gtk.PositionType)(2));
        this.table2.Add(this.tintAmountScale);
        Gtk.Table.TableChild w17 = ((Gtk.Table.TableChild)(this.table2[this.tintAmountScale]));
        w17.TopAttach = ((uint)(1));
        w17.BottomAttach = ((uint)(2));
        w17.LeftAttach = ((uint)(1));
        w17.RightAttach = ((uint)(2));
        w17.YOptions = ((Gtk.AttachOptions)(4));
        // Container child table2.Gtk.Table+TableChild
        this.tintHueScale = new Gtk.HScale(null);
        this.tintHueScale.CanFocus = true;
        this.tintHueScale.Name = "tintHueScale";
        this.tintHueScale.Adjustment.Upper = 360;
        this.tintHueScale.Adjustment.PageIncrement = 10;
        this.tintHueScale.Adjustment.StepIncrement = 1;
        this.tintHueScale.Adjustment.Value = 23;
        this.tintHueScale.DrawValue = true;
        this.tintHueScale.Digits = 0;
        this.tintHueScale.ValuePos = ((Gtk.PositionType)(2));
        this.table2.Add(this.tintHueScale);
        Gtk.Table.TableChild w18 = ((Gtk.Table.TableChild)(this.table2[this.tintHueScale]));
        w18.LeftAttach = ((uint)(1));
        w18.RightAttach = ((uint)(2));
        w18.YOptions = ((Gtk.AttachOptions)(4));
        this.controlsVBox.Add(this.table2);
        Gtk.Box.BoxChild w19 = ((Gtk.Box.BoxChild)(this.controlsVBox[this.table2]));
        w19.PackType = ((Gtk.PackType)(1));
        w19.Position = 4;
        w19.Expand = false;
        w19.Fill = false;
        this.paned.Add(this.controlsVBox);
        Gtk.Paned.PanedChild w20 = ((Gtk.Paned.PanedChild)(this.paned[this.controlsVBox]));
        w20.Resize = false;
        this.vbox3.Add(this.paned);
        Gtk.Box.BoxChild w21 = ((Gtk.Box.BoxChild)(this.vbox3[this.paned]));
        w21.Position = 1;
        this.Add(this.vbox3);
        if ((this.Child != null)) {
            this.Child.ShowAll();
        }
        this.DefaultWidth = 724;
        this.DefaultHeight = 588;
        this.Show();
        this.DeleteEvent += new Gtk.DeleteEventHandler(this.OnDeleteEvent);
        this.Open1.Activated += new System.EventHandler(this.OpenFile);
        this.Export.Activated += new System.EventHandler(this.ExportFile);
        this.SaveAs.Activated += new System.EventHandler(this.SaveFileAs);
        this.Import.Activated += new System.EventHandler(this.ImportFile);
        this.ZoomIn.Activated += new System.EventHandler(this.DoZoomIn);
        this.ZoomOut.Activated += new System.EventHandler(this.DoZoomOut);
        this.redScale.ValueChanged += new System.EventHandler(this.MixerValueChanged);
        this.greenScale.ValueChanged += new System.EventHandler(this.MixerValueChanged);
        this.blueScale.ValueChanged += new System.EventHandler(this.MixerValueChanged);
        this.tintHueScale.ValueChanged += new System.EventHandler(this.TintChanged);
        this.tintAmountScale.ValueChanged += new System.EventHandler(this.TintChanged);
    }
}
